@@ -36,13 +36,12 @@ diagnostic log of the segment creation in grSegmentLog.txt
 #include <climits>
 #include <iomanip>
 #include <cstring>
-#ifdef HAVE_ICONV
-#include <iconv.h>
-#endif
+
+#include "UtfCodec.h"
 
 #include "graphite2/Types.h"
 #include "graphite2/Segment.h"
-#include "graphite2/XmlLog.h"
+#include "graphite2/Log.h"
 
 class Gr2TextSrc
 {
@@ -93,8 +92,8 @@ public:
     bool ws;
     bool rtl;
     bool useLineFill;
-    bool useCodes;
-    bool justification;
+    int useCodes;
+    int justification;
     bool enableCache;
     float width;
     int textArgIndex;
@@ -102,8 +101,9 @@ public:
     size_t charLength;
     size_t offset;
     FILE * log;
-    FILE * trace;
-    int mask;
+    char * trace;
+    int codesize;
+    gr_face_options opts;
     
 private :  //defensive since log should not be copied
     Parameters(const Parameters&);
@@ -135,17 +135,18 @@ void Parameters::clear()
     rtl = false;
     ws = false;
     useLineFill = false;
-    useCodes = false;
-    justification = false;
+    useCodes = 0;
+    justification = 0;
     enableCache = false;
     width = 100.0f;
     pText32 = NULL;
     textArgIndex = 0;
     charLength = 0;
+    codesize = 4;
     offset = 0;
     log = stdout;
     trace = NULL;
-    mask = GRLOG_SEGMENT | GRLOG_OPCODE;
+    opts = gr_face_preloadAll;
 }
 
 
@@ -160,56 +161,33 @@ void Parameters::closeLog()
 
 int lookup(size_t *map, size_t val);
 
+namespace gr2 = graphite2;
 
-#ifdef HAVE_ICONV
-
-template <class A,class B> int
-convertUtf(const char * inType, const char * outType, A* pIn, B * & pOut)
+template <typename utf>
+size_t convertUtf(const void * src, unsigned int * & dest)
 {
-    int length = 0;//strlen(reinterpret_cast<char*>(pIn));
-    while (pIn[length] != 0) length++;
-    int outFactor = 1;
-    if (sizeof(B) < sizeof(A))
-    {
-        outFactor = sizeof(A) / sizeof(B);
-    }
-#ifdef WIN32
-    const char * pText = reinterpret_cast<char*>(pIn);
-#else
-    char * pText = reinterpret_cast<char*>(pIn);
-#endif
-    // It seems to be necessary to include the trailing null to prevent
-    // stray characters appearing with utf16
-    size_t bytesLeft = (length+1) * sizeof(A);
-    // allow 2 extra for null + bom
-    size_t outBytesLeft = (length*outFactor + 2) * sizeof(B);
-    size_t outBufferSize = outBytesLeft;
-    //B * textOut = new B[length*outFactor + 2];
-    B * textOut = reinterpret_cast<B*>(malloc(sizeof(B) * length*outFactor + 2));// new operator not defined
-    iconv_t utfInOut = iconv_open(outType,inType);
-    assert(utfInOut != (iconv_t)(-1));
-    char * pTextOut = reinterpret_cast<char*>(&textOut[0]);
-    size_t status = iconv(utfInOut, &pText, &bytesLeft, &pTextOut, &outBytesLeft);
-    if (status == size_t(-1)) perror("iconv failed:");
-    size_t charLength = (outBufferSize - outBytesLeft) / sizeof(B);
-    assert(status != size_t(-1));
-    // offset by 1 to avoid bom
-    unsigned char * bom = reinterpret_cast<unsigned char*>(&textOut[0]);
-    if ((charLength * sizeof(B) > 1) && ((bom[1] == 0xfe && bom[0] == 0xff)
-        || (bom[0] == 0xfe && bom[1] == 0xff)))
-    {
-            charLength--;
-            for (size_t i = 0; i < charLength; i++)
-                textOut[i] = textOut[i+1];
-            textOut[charLength] = 0;
-    }
-    if (textOut[charLength] == 0) --charLength;
-    pOut = textOut;
-    iconv_close(utfInOut);
-    return charLength;
+    dest = static_cast<unsigned int *>(malloc(sizeof(unsigned int)*(strlen(reinterpret_cast<const char *>(src))+1)));
+    if (!dest)
+    	return 0;
+
+    typename utf::const_iterator ui = src;
+    unsigned int * out = dest;
+	while ((*out = *ui) != 0 && !ui.error())
+	{
+		++ui;
+		++out;
+	}
+
+	if (ui.error())
+	{
+		free(dest);
+		dest = 0;
+		return size_t(-1);
+	}
+
+	return (out-dest);
 }
 
-#endif
 
 bool Parameters::loadFromArgs(int argc, char *argv[])
 {
@@ -217,6 +195,7 @@ bool Parameters::loadFromArgs(int argc, char *argv[])
     pText32 = NULL;
     features = NULL;
     log = stdout;
+    codesize = 4;
     bool argError = false;
     char* pText = NULL;
     typedef enum 
@@ -224,6 +203,7 @@ bool Parameters::loadFromArgs(int argc, char *argv[])
         NONE,
         POINT_SIZE,
         DPI,
+        JUSTIFY,
         LINE_START,
         LINE_END,
         LINE_FILL,
@@ -231,7 +211,7 @@ bool Parameters::loadFromArgs(int argc, char *argv[])
         FEAT,
         LOG,
         TRACE,
-        TRACE_MASK
+        SIZE
     } TestOptions;
     TestOptions option = NONE;
     char * pIntEnd = NULL;
@@ -270,6 +250,16 @@ bool Parameters::loadFromArgs(int argc, char *argv[])
             }
             option = NONE;
             break;
+        case JUSTIFY:
+            pIntEnd = NULL;
+            justification = strtol(argv[a], &pIntEnd, 10);
+            if (justification <= 0)
+            {
+                fprintf(stderr, "Justification value must be > 0 but was %s\n", argv[a]);
+                justification = 0;
+            }
+            option = NONE;
+            break;
         case LINE_FILL:
             pFloatEnd = NULL;
             fTestSize = strtof(argv[a],&pFloatEnd);
@@ -300,16 +290,12 @@ bool Parameters::loadFromArgs(int argc, char *argv[])
             option = NONE;
             break;
         case TRACE:
-            if (trace) fclose(trace);
-            trace = fopen(argv[a], "wb");
-            if (trace == NULL)
-            {
-                fprintf(stderr,"Failed to open %s\n", argv[a]);
-            }
+            trace = argv[a];
             option = NONE;
             break;
-        case TRACE_MASK:
-            mask = atoi(argv[a]);
+        case SIZE :
+            pIntEnd = NULL;
+            codesize = strtol(argv[a],&pIntEnd, 10);
             option = NONE;
             break;
         default:
@@ -358,10 +344,14 @@ bool Parameters::loadFromArgs(int argc, char *argv[])
                 {
                     option = FEAT;
                 }
+                else if (strcmp(argv[a], "-bytes") == 0)
+                {
+                    option = SIZE;
+                }
                 else if (strcmp(argv[a], "-codes") == 0)
                 {
                     option = NONE;
-                    useCodes = true;
+                    useCodes = 4;
                     // must be less than argc
                     //pText32 = new unsigned int[argc];
                     pText32 = (unsigned int *)malloc(sizeof(unsigned int) * argc);
@@ -374,8 +364,7 @@ bool Parameters::loadFromArgs(int argc, char *argv[])
                 }
                 else if (strcmp(argv[a], "-j") == 0)
                 {
-                    option = NONE;
-                    justification = true;
+                    option = JUSTIFY;
                 }
                 else if (strcmp(argv[a], "-log") == 0)
                 {
@@ -385,9 +374,10 @@ bool Parameters::loadFromArgs(int argc, char *argv[])
                 {
                     option = TRACE;
                 }
-                else if (strcmp(argv[a], "-mask") == 0)
+                else if (strcmp(argv[a], "-demand") == 0)
                 {
-                    option = TRACE_MASK;
+                    option = NONE;
+                    opts = gr_face_default;
                 }
                 else
                 {
@@ -438,8 +428,14 @@ bool Parameters::loadFromArgs(int argc, char *argv[])
     {
         if (!useCodes && pText != NULL)
         {
-#ifdef HAVE_ICONV
-            charLength = convertUtf("utf-8","utf-32",pText, pText32);
+            charLength = convertUtf<gr2::utf8>(pText, pText32);
+            if (!pText32)
+            {
+            	if (charLength == -1)
+            		perror("decoding utf-8 data failed");
+            	else
+            		perror("insufficent memory for text buffer");
+            }
             fprintf(log, "String has %d characters\n", (int)charLength);
             size_t ci;
             for (ci = 0; ci < 10 && ci < charLength; ci++)
@@ -454,10 +450,6 @@ bool Parameters::loadFromArgs(int argc, char *argv[])
                         fprintf(log, "\n");
             }
             fprintf(log, "\n");
-#else
-            fprintf(stderr,"Only the -codes option is supported on Win32\n");
-            argError = true;
-#endif
         }
         else 
         {
@@ -606,16 +598,14 @@ int Parameters::testFileFont() const
     int returnCode = 0;
 //    try
     {
-        // use the -trace option to specify a file
-#ifndef DISABLE_TRACING
-        graphite_start_logging(trace, static_cast<GrLogMask>(mask));
-#endif
-
         gr_face *face = NULL;
         if (enableCache)
-            face = gr_make_file_face_with_seg_cache(fileName, 1000, gr_face_preloadGlyphs | gr_face_dumbRendering);
+            face = gr_make_file_face_with_seg_cache(fileName, 1000, opts | gr_face_dumbRendering);
         else
-            face = gr_make_file_face(fileName, gr_face_preloadGlyphs);
+            face = gr_make_file_face(fileName, opts);
+
+        // use the -trace option to specify a file
+    	if (trace)	gr_start_logging(face, trace);
 
         if (!face)
         {
@@ -625,8 +615,8 @@ int Parameters::testFileFont() const
         if (charLength == 0)
         {
             printFeatures(face);
+            gr_stop_logging(face);
             gr_face_destroy(face);
-            graphite_stop_logging();
             return 0;
         }
 
@@ -676,101 +666,118 @@ int Parameters::testFileFont() const
        {
         gr_segment* pSeg = NULL;
         if (features)
-        {
             featureList = parseFeatures(face);
-            pSeg = gr_make_seg(sizedFont,
-                face, 0, featureList, textSrc.utfEncodingForm(),
-                textSrc.get_utf_buffer_begin(), textSrc.getLength(), rtl ? 1 : 0);
+        if (codesize == 2)
+        {
+            unsigned short *pText16 = (unsigned short *)malloc(textSrc.getLength() * 2 * sizeof(unsigned short));
+            gr2::utf16::iterator ui = pText16;
+            unsigned int *p = pText32;
+            for (int i = 0; i < textSrc.getLength(); ++i)
+            {
+                *ui = *p++;
+                ui++;
+            }
+            *ui = 0;
+            pSeg = gr_make_seg(sizedFont, face, 0, features ? featureList : NULL, (gr_encform)codesize, pText16, textSrc.getLength(), rtl ? 1 : 0);
+        }
+        else if (codesize == 1)
+        {
+            unsigned char *pText8 = (unsigned char *)malloc(textSrc.getLength() * 4);
+            gr2::utf8::iterator ui = pText8;
+            unsigned int *p = pText32;
+            for (int i = 0; i < textSrc.getLength(); ++i)
+            {
+                *ui = *p++;
+                ui++;
+            }
+            *ui = 0;
+            pSeg = gr_make_seg(sizedFont, face, 0, features ? featureList : NULL, (gr_encform)codesize, pText8, textSrc.getLength(), rtl ? 1 : 0);
         }
         else
-        {
-            pSeg = gr_make_seg(sizedFont, face, 0, NULL, textSrc.utfEncodingForm(),
+            pSeg = gr_make_seg(sizedFont, face, 0, features ? featureList : NULL, textSrc.utfEncodingForm(),
                 textSrc.get_utf_buffer_begin(), textSrc.getLength(), rtl ? 1 : 0);
-        }
-        int i = 0;
-#ifndef NDEBUG
-        int numSlots = gr_seg_n_slots(pSeg);
-#endif
-//        size_t *map = new size_t [seg.length() + 1];
-        size_t *map = (size_t*)malloc((gr_seg_n_slots(pSeg) + 1) * sizeof(size_t));
-        for (const gr_slot* slot = gr_seg_first_slot(pSeg); slot; slot = gr_slot_next_in_segment(slot), ++i)
-        { map[i] = (size_t)slot; }
-        map[i] = 0;
-        fprintf(log, "pos  gid   attach\t     x\t     y\tins bw\t  chars\t\tUnicode\t");
-        fprintf(log, "\n");
-        i = 0;
-        for (const gr_slot* slot = gr_seg_first_slot(pSeg); slot; slot = gr_slot_next_in_segment(slot), ++i)
+
+        if (pSeg)
         {
-            // consistency check for last slot
-            assert((i + 1 < numSlots) || (slot == gr_seg_last_slot(pSeg)));
-            float orgX = gr_slot_origin_X(slot);
-            float orgY = gr_slot_origin_Y(slot);
-            fprintf(log, "%02d  %4d %3d@%d,%d\t%6.1f\t%6.1f\t%2d%4d\t%3d %3d\t",
-                    i, gr_slot_gid(slot), lookup(map, (size_t)gr_slot_attached_to(slot)),
-                    gr_slot_attr(slot, pSeg, gr_slatAttX, 0),
-                    gr_slot_attr(slot, pSeg, gr_slatAttY, 0), orgX, orgY, gr_slot_can_insert_before(slot) ? 1 : 0,
-                    gr_cinfo_break_weight(gr_seg_cinfo(pSeg, gr_slot_original(slot))), gr_slot_before(slot), gr_slot_after(slot));
-           
-            if (pText32 != NULL)
-            {
-                fprintf(log, "%7x\t%7x",
-                    pText32[gr_slot_before(slot) + offset],
-                    pText32[gr_slot_after(slot) + offset]);
-            }
-#if 0
-            if (parameters.justification)
-            {
-                // only level 0 seems to be supported without an assertion
-                for (int level = 0; level < 1; level++)
-                {
-                    printf("\t% 2d %6.1f %6.1f %6.1f %d", level,
-                        info.maxShrink(level),
-                        info.maxStretch(level), info.stretchStep(level),
-                        info.justWeight(level));
-                    if (info.maxShrink(level) == 0.0f &&
-                        info.maxStretch(level) == 0.0f &&
-                        info.stretchStep(level) == 0.0f &&
-                        info.justWeight(level) == 0)
-                        break;
-                }
-            }
-            printf("\n");
-            ++i;
-            ++gi;
-#endif
+            int i = 0;
+            float advanceWidth;
+    #ifndef NDEBUG
+            int numSlots = gr_seg_n_slots(pSeg);
+    #endif
+    //        size_t *map = new size_t [seg.length() + 1];
+            if (justification > 0)
+                advanceWidth = gr_seg_justify(pSeg, gr_seg_first_slot(pSeg), sizedFont, gr_seg_advance_X(pSeg) * justification / 100., gr_justCompleteLine, NULL, NULL);
+            else
+                advanceWidth = gr_seg_advance_X(pSeg);
+            size_t *map = (size_t*)malloc((gr_seg_n_slots(pSeg) + 1) * sizeof(size_t));
+            for (const gr_slot* slot = gr_seg_first_slot(pSeg); slot; slot = gr_slot_next_in_segment(slot), ++i)
+            { map[i] = (size_t)slot; }
+            map[i] = 0;
+            fprintf(log, "Segment length: %d\n", gr_seg_n_slots(pSeg));
+            fprintf(log, "pos  gid   attach\t     x\t     y\tins bw\t  chars\t\tUnicode\t");
             fprintf(log, "\n");
+            i = 0;
+            for (const gr_slot* slot = gr_seg_first_slot(pSeg); slot; slot = gr_slot_next_in_segment(slot), ++i)
+            {
+                // consistency check for last slot
+                assert((i + 1 < numSlots) || (slot == gr_seg_last_slot(pSeg)));
+                float orgX = gr_slot_origin_X(slot);
+                float orgY = gr_slot_origin_Y(slot);
+                fprintf(log, "%02d  %4d %3d@%d,%d\t%6.1f\t%6.1f\t%2d%4d\t%3d %3d\t",
+                        i, gr_slot_gid(slot), lookup(map, (size_t)gr_slot_attached_to(slot)),
+                        gr_slot_attr(slot, pSeg, gr_slatAttX, 0),
+                        gr_slot_attr(slot, pSeg, gr_slatAttY, 0), orgX, orgY, gr_slot_can_insert_before(slot) ? 1 : 0,
+                        gr_cinfo_break_weight(gr_seg_cinfo(pSeg, gr_slot_original(slot))), gr_slot_before(slot), gr_slot_after(slot));
+               
+                if (pText32 != NULL)
+                {
+                    fprintf(log, "%7x\t%7x",
+                        pText32[gr_slot_before(slot) + offset],
+                        pText32[gr_slot_after(slot) + offset]);
+                }
+    #if 0
+                if (parameters.justification)
+                {
+                    // only level 0 seems to be supported without an assertion
+                    for (int level = 0; level < 1; level++)
+                    {
+                        printf("\t% 2d %6.1f %6.1f %6.1f %d", level,
+                            info.maxShrink(level),
+                            info.maxStretch(level), info.stretchStep(level),
+                            info.justWeight(level));
+                        if (info.maxShrink(level) == 0.0f &&
+                            info.maxStretch(level) == 0.0f &&
+                            info.stretchStep(level) == 0.0f &&
+                            info.justWeight(level) == 0)
+                            break;
+                    }
+                }
+                printf("\n");
+                ++i;
+                ++gi;
+    #endif
+                fprintf(log, "\n");
+            }
+            assert(i == numSlots);
+            // assign last point to specify advance of the whole array
+            // position arrays must be one bigger than what countGlyphs() returned
+            fprintf(log, "Advance width = %6.1f\n", advanceWidth);
+            unsigned int numchar = gr_seg_n_cinfo(pSeg);
+            fprintf(log, "\nChar\tUnicode\tBefore\tAfter\n");
+            for (unsigned int j = 0; j < numchar; j++)
+            {
+                const gr_char_info *c = gr_seg_cinfo(pSeg, j);
+                fprintf(log, "%d\t%04X\t%d\t%d\n", j, gr_cinfo_unicode_char(c), gr_cinfo_before(c), gr_cinfo_after(c));
+            }
+            free(map);
+            gr_seg_destroy(pSeg);
         }
-        assert(i == numSlots);
-        // assign last point to specify advance of the whole array
-        // position arrays must be one bigger than what countGlyphs() returned
-        float advanceWidth = gr_seg_advance_X(pSeg);
-        fprintf(log, "Advance width = %6.1f\n", advanceWidth);
-        unsigned int numchar = gr_seg_n_cinfo(pSeg);
-        fprintf(log, "\nChar\tUnicode\tBefore\tAfter\n");
-        for (unsigned int j = 0; j < numchar; j++)
-        {
-            const gr_char_info *c = gr_seg_cinfo(pSeg, j);
-            fprintf(log, "%d\t%04X\t%d\t%d\n", j, gr_cinfo_unicode_char(c), gr_cinfo_before(c), gr_cinfo_after(c));
-        }
-        free(map);
-        gr_seg_destroy(pSeg);
        }
         if (featureList) gr_featureval_destroy(featureList);
         gr_font_destroy(sizedFont);
+        if (trace) gr_stop_logging(face);
         gr_face_destroy(face);
-//            delete featureParser;
-        // setText copies the text, so it is no longer needed
-//        delete [] parameters.pText32;
-//        logStream.close();
     }
-//    catch (...)
-//    {
-//        printf("Exception occurred\n");
-//        returnCode = 5;
-//    }
-#ifndef DISABLE_TRACING
-    if (trace) graphite_stop_logging();
-#endif
     return returnCode;
 }
 
@@ -797,14 +804,16 @@ int main(int argc, char *argv[])
         //fprintf(stderr,"-ls\tStart of line = true (false)\n");
         //fprintf(stderr,"-le\tEnd of line = true (false)\n");
         fprintf(stderr,"-rtl\tRight to left = true (false)\n");
+        fprintf(stderr,"-j percentage\tJustify to percentage of string width\n");
         //fprintf(stderr,"-ws\tAllow trailing whitespace = true (false)\n");
         //fprintf(stderr,"-linefill w\tuse a LineFillSegment of width w (RangeSegment)\n");
         fprintf(stderr,"\nIf a font, but no text is specified, then a list of features will be shown.\n");
         fprintf(stderr,"-feat f=g\tSet feature f to value g. Separate multiple features with ,\n");
         fprintf(stderr,"-log out.log\tSet log file to use rather than stdout\n");
         fprintf(stderr,"-trace trace.xml\tDefine a file for the XML trace log\n");
-        fprintf(stderr,"-mask mask\tDefine the mask to use for trace logging\n");
+        fprintf(stderr,"-demand\tDemand load glyphs and cmap cache\n");
         fprintf(stderr,"-cache\tEnable Segment Cache\n");
+        fprintf(stderr,"-bytes\tword size for character transfer [1,2,4] defaults to 4\n");
         return 1;
     }
     return parameters.testFileFont();
