@@ -33,8 +33,7 @@
 #include "libcdr_utils.h"
 
 libcdr::CDRParserState::CDRParserState()
-  : m_fillStyles(), m_lineStyles(), m_bmps(), m_patterns(), m_vects(), m_pages(),
-    m_documentPalette(), m_fonts(), m_texts(),
+  : m_bmps(), m_patterns(), m_vects(), m_pages(), m_documentPalette(), m_texts(),
     m_colorTransformCMYK2RGB(0), m_colorTransformLab2RGB(0), m_colorTransformRGB2RGB(0)
 {
   cmsHPROFILE tmpRGBProfile = cmsCreate_sRGBProfile();
@@ -62,7 +61,9 @@ void libcdr::CDRParserState::setColorTransform(const std::vector<unsigned char>
 {
   if (profile.empty())
     return;
-  cmsHPROFILE tmpProfile = cmsOpenProfileFromMem(&profile[0], profile.size());
+  cmsHPROFILE tmpProfile = cmsOpenProfileFromMem(&profile[0], cmsUInt32Number(profile.size()));
+  if (!tmpProfile)
+    return;
   cmsHPROFILE tmpRGBProfile = cmsCreate_sRGBProfile();
   cmsColorSpaceSignature signature = cmsGetColorSpace(tmpProfile);
   switch (signature)
@@ -322,7 +323,7 @@ unsigned libcdr::CDRParserState::_getRGBColor(const CDRColor &color)
       0x00, 0x46, 0x73, 0x91, 0xa2, 0xad, 0x2a, 0x3d, 0x69, 0x7d, 0x85, 0x8f, 0xa3,
     };
 
-    unsigned short pantoneIndex = ((unsigned short)col1 << 8) | (unsigned short)col0;
+    unsigned short pantoneIndex = (unsigned short)(((int)col1 << 8) | (int)col0);
     double pantoneSaturation = (double)(((unsigned short)col3 << 8) | (unsigned short)col2) / 100.0;
     typedef struct
     {
@@ -339,9 +340,9 @@ unsigned libcdr::CDRParserState::_getRGBColor(const CDRColor &color)
       pureColor.g = WaldoColorType0_G[pantoneIndex];
       pureColor.b = WaldoColorType0_B[pantoneIndex];
     }
-    unsigned tmpRed = cdr_round(255.0*(1-pantoneSaturation) + (double)pureColor.r*pantoneSaturation);
-    unsigned tmpGreen = cdr_round(255.0*(1-pantoneSaturation) + (double)pureColor.g*pantoneSaturation);
-    unsigned tmpBlue = cdr_round(255.0*(1-pantoneSaturation) + (double)pureColor.b*pantoneSaturation);
+    unsigned tmpRed = (unsigned)cdr_round(255.0*(1-pantoneSaturation) + (double)pureColor.r*pantoneSaturation);
+    unsigned tmpGreen = (unsigned)cdr_round(255.0*(1-pantoneSaturation) + (double)pureColor.g*pantoneSaturation);
+    unsigned tmpBlue = (unsigned)cdr_round(255.0*(1-pantoneSaturation) + (double)pureColor.b*pantoneSaturation);
     red = (tmpRed < 255 ? (unsigned char)tmpRed : 255);
     green = (tmpGreen < 255 ? (unsigned char)tmpGreen : 255);
     blue = (tmpBlue < 255 ? (unsigned char)tmpBlue : 255);
@@ -407,7 +408,7 @@ unsigned libcdr::CDRParserState::_getRGBColor(const CDRColor &color)
   // HSB
   case 0x06:
   {
-    unsigned short hue = (col1<<8) | col0;
+    unsigned short hue = (unsigned short)(((int)col1<<8) | col0);
     double saturation = (double)col2/255.0;
     double brightness = (double)col3/255.0;
 
@@ -442,7 +443,7 @@ unsigned libcdr::CDRParserState::_getRGBColor(const CDRColor &color)
   // HLS
   case 0x07:
   {
-    unsigned short hue = (col1<<8) | col0;
+    unsigned short hue = (unsigned short)(((int)col1<<8) | col0);
     double lightness = (double)col2/255.0;
     double saturation = (double)col3/255.0;
 
@@ -453,7 +454,7 @@ unsigned libcdr::CDRParserState::_getRGBColor(const CDRColor &color)
 
     if (hue < 120)
     {
-      satRed =  (double)(120 - hue) / 60.0;
+      satRed = (double)(120 - hue) / 60.0;
       satGreen = (double)hue/60.0;
       satBlue = 0.0;
     }
@@ -496,6 +497,55 @@ unsigned libcdr::CDRParserState::_getRGBColor(const CDRColor &color)
     blue = col0;
     break;
   }
+  // YIQ255
+  case 0x0b:
+  {
+    double y = (double)col0;
+    double i = (double)col1;
+    double q = (double)col2;
+
+    y -= 100.0;
+    if (y < 0.0)
+      y /= 100.0;
+    else
+      y /= 155.0;
+    y *= 0.5;
+    y += 0.5;
+
+    i -= 100.0;
+    if (i <= 0.0)
+      i /= 100.0;
+    else
+      i /= 155;
+    i *= 0.5957;
+
+    q -= 100.0;
+    if (q <= 0)
+      q /= 100.0;
+    else
+      q /= 155;
+    q *= 0.5226;
+
+    double RR = y + 0.9563*i + 0.6210*q;
+    double GG = y - 0.2127*i - 0.6474*q;
+    double BB = y - 1.1070*i + 1.7046*q;
+    if (RR > 1.0)
+      RR = 1.0;
+    if (RR < 0.0)
+      RR = 0.0;
+    if (GG > 1.0)
+      GG = 1.0;
+    if (GG < 0.0)
+      GG = 0.0;
+    if (BB > 1.0)
+      BB = 1.0;
+    if (BB < 0.0)
+      BB = 0.0;
+    red = (unsigned char)cdr_round(255*RR);
+    green = (unsigned char)cdr_round(255*GG);
+    blue = (unsigned char)cdr_round(255*BB);
+    break;
+  }
   // Lab
   case 0x0c:
   {
