@@ -33,20 +33,21 @@
 #include <math.h>
 #include <libwpd/libwpd.h>
 #include "CDRTransforms.h"
+#include "CDRPath.h"
+#include "libcdr_utils.h"
 
 namespace libcdr
 {
-class CDRPath;
 
-struct CDRBBox
+struct CDRBox
 {
   double m_x;
   double m_y;
   double m_w;
   double m_h;
-  CDRBBox()
+  CDRBox()
     : m_x(0.0), m_y(0.0), m_w(0.0), m_h(0.0) {}
-  CDRBBox(double x0, double y0, double x1, double y1)
+  CDRBox(double x0, double y0, double x1, double y1)
     : m_x(x0 < x1 ? x0 : x1), m_y(y0 < y1 ? y0 : y1), m_w(fabs(x1-x0)), m_h(fabs(y1-y0)) {}
   double getWidth() const
   {
@@ -122,7 +123,7 @@ struct CDRFillStyle
   CDRGradient gradient;
   CDRImageFill imageFill;
   CDRFillStyle()
-    : fillType(0), color1(), color2(), gradient(), imageFill() {}
+    : fillType((unsigned short)-1), color1(), color2(), gradient(), imageFill() {}
   CDRFillStyle(unsigned short ft, CDRColor c1, CDRColor c2, const CDRGradient &gr, const CDRImageFill &img)
     : fillType(ft), color1(c1), color2(c2), gradient(gr), imageFill(img) {}
 };
@@ -137,37 +138,58 @@ struct CDRLineStyle
   double angle;
   CDRColor color;
   std::vector<unsigned> dashArray;
-  unsigned startMarkerId;
-  unsigned endMarkerId;
+  CDRPath startMarker;
+  CDRPath endMarker;
   CDRLineStyle()
-    : lineType(0), capsType(0), joinType(0), lineWidth(0.0),
+    : lineType((unsigned short)-1), capsType(0), joinType(0), lineWidth(0.0),
       stretch(0.0), angle(0.0), color(), dashArray(),
-      startMarkerId(0), endMarkerId(0) {}
+      startMarker(), endMarker() {}
   CDRLineStyle(unsigned short lt, unsigned short ct, unsigned short jt,
                double lw, double st, double a, const CDRColor &c, const std::vector<unsigned> &da,
-               unsigned smi, unsigned emi)
+               const CDRPath &sm, const CDRPath &em)
     : lineType(lt), capsType(ct), joinType(jt), lineWidth(lw),
       stretch(st), angle(a), color(c), dashArray(da),
-      startMarkerId(smi), endMarkerId(emi) {}
+      startMarker(sm), endMarker(em) {}
 };
 
 struct CDRCharacterStyle
 {
   unsigned short m_charSet;
-  unsigned short m_fontId;
+  WPXString m_fontName;
   double m_fontSize;
+  unsigned m_align;
+  double m_leftIndent, m_firstIndent, m_rightIndent;
+  CDRLineStyle m_lineStyle;
+  CDRFillStyle m_fillStyle;
+  unsigned m_parentId;
   CDRCharacterStyle()
-    : m_charSet(0), m_fontId(0), m_fontSize(0.0) {}
-  CDRCharacterStyle(unsigned short charSet, unsigned short fontId, double fontSize)
-    : m_charSet(charSet), m_fontId(fontId), m_fontSize(fontSize) {}
+    : m_charSet((unsigned short)-1), m_fontName(),
+      m_fontSize(0.0), m_align(0), m_leftIndent(0.0), m_firstIndent(0.0),
+      m_rightIndent(0.0), m_lineStyle(), m_fillStyle(), m_parentId(0)
+  {
+    m_fontName.clear();
+  }
   void overrideCharacterStyle(const CDRCharacterStyle &override)
   {
-    if (override.m_charSet)
+    if (override.m_charSet != (unsigned short)-1 || override.m_fontName.len())
+    {
       m_charSet = override.m_charSet;
-    if (override.m_fontId)
-      m_fontId = override.m_fontId;
-    if (override.m_fontSize > 0.0)
+      m_fontName = override.m_fontName;
+    }
+    if (!CDR_ALMOST_ZERO(override.m_fontSize))
       m_fontSize = override.m_fontSize;
+    if (override.m_align)
+      m_align = override.m_align;
+    if (override.m_leftIndent != 0.0 && override.m_firstIndent != 0.0 && override.m_rightIndent != 0.0)
+    {
+      m_leftIndent = override.m_leftIndent;
+      m_firstIndent = override.m_firstIndent;
+      m_rightIndent = override.m_rightIndent;
+    }
+    if (override.m_lineStyle.lineType != (unsigned short)-1)
+      m_lineStyle = override.m_lineStyle;
+    if (override.m_fillStyle.fillType != (unsigned short)-1)
+      m_fillStyle = override.m_fillStyle;
   }
 };
 
@@ -344,6 +366,21 @@ struct CDRText
   CDRCharacterStyle m_charStyle;
 };
 
+struct CDRTextLine
+{
+  CDRTextLine() : m_line() {}
+  CDRTextLine(const CDRTextLine &line) : m_line(line.m_line) {}
+  void append(const CDRText &text)
+  {
+    m_line.push_back(text);
+  }
+  void clear()
+  {
+    m_line.clear();
+  }
+  std::vector<CDRText> m_line;
+};
+
 struct CDRFont
 {
   CDRFont() : m_name(), m_encoding(0) {}
